"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.querySchemaMixin = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const constants_1 = require("../../../constants");
const types_1 = require("../../../types");
const helpers_1 = require("../../../helpers");
function querySchemaMixin(entity) {
    let QuerySchemaMixin = class QuerySchemaMixin {
        async transform(value) {
            const builtQuery = {
                ["sort" /* sort */]: undefined,
                ["filter" /* filter */]: undefined,
                ["include" /* include */]: undefined,
                ["page" /* page */]: {
                    number: constants_1.DEFAULT_QUERY_PAGE,
                    size: constants_1.DEFAULT_PAGE_SIZE,
                }
            };
            if (!value) {
                return builtQuery;
            }
            if (value["filter" /* filter */]) {
                builtQuery["filter" /* filter */] = Object
                    .entries(value["filter" /* filter */])
                    .reduce((accum, [field, value]) => {
                    if (value[types_1.FilterOperand.in] || value[types_1.FilterOperand.nin]) {
                        accum[field] = Object.entries(value).reduce((acum, [op, filed]) => {
                            acum[op] = helpers_1.isString(filed) ? filed.split(',') : [];
                            return acum;
                        }, {});
                    }
                    else if (helpers_1.isString(value)) {
                        accum[field] = {
                            [types_1.FilterOperand.eq]: value,
                        };
                    }
                    else {
                        accum[field] = value;
                    }
                    return accum;
                }, {});
            }
            if (value["include" /* include */] && helpers_1.isString(value["include" /* include */])) {
                builtQuery["include" /* include */] = value["include" /* include */].split(',');
            }
            if (value["sort" /* sort */] && helpers_1.isString(value["sort" /* sort */])) {
                builtQuery["sort" /* sort */] = value["sort" /* sort */].split(',')
                    .reduce((acum, field) => {
                    const fieldName = field.charAt(0) === '-' ? field.substring(1) : field;
                    const sortType = field.charAt(0) === '-' ? "DESC" /* DESC */ : "ASC" /* ASC */;
                    acum[fieldName] = sortType;
                    return acum;
                }, {});
            }
            if (value["page" /* page */]) {
                const { number, size } = value["page" /* page */];
                if (number) {
                    builtQuery["page" /* page */].number = parseInt(number, 10);
                }
                else {
                    builtQuery["page" /* page */].number = constants_1.DEFAULT_QUERY_PAGE;
                }
                if (size) {
                    builtQuery["page" /* page */].size = parseInt(size, 10);
                }
                else {
                    builtQuery["page" /* page */].size = constants_1.DEFAULT_PAGE_SIZE;
                }
            }
            const errors = await helpers_1.checkQueryParamsSchema(builtQuery);
            if (errors.length > 0) {
                throw new common_1.BadRequestException(errors);
            }
            return {
                ["include" /* include */]: builtQuery["include" /* include */] || [],
                ["filter" /* filter */]: builtQuery["filter" /* filter */] || {},
                ["sort" /* sort */]: builtQuery["sort" /* sort */] || {},
                ["page" /* page */]: builtQuery["page" /* page */]
            };
        }
    };
    QuerySchemaMixin = tslib_1.__decorate([
        common_1.Injectable()
    ], QuerySchemaMixin);
    return helpers_1.mixin(QuerySchemaMixin);
}
exports.querySchemaMixin = querySchemaMixin;
//# sourceMappingURL=query-schema.js.map